#include "ds-devices.h"

#include <ds-model/model.h>

using namespace std::literals;
static ds::LogChannel dsLogChannel("dsApartmentDsDevices");

std::optional<dsModel::OutputType> DSToEnum<dsModel::OutputType, dss::ChannelType, void>::tryTo(dss::ChannelType x) {
  switch (x) {
    case dss::ChannelType::none: return {};
    case dss::ChannelType::brightness: return dsModel::OutputType::lightBrightness;
    case dss::ChannelType::hue: return dsModel::OutputType::lightHue;
    case dss::ChannelType::saturation: return dsModel::OutputType::lightSaturation;
    case dss::ChannelType::colortemp: return dsModel::OutputType::lightTemperature;
    case dss::ChannelType::x: return dsModel::OutputType::lightCieX;
    case dss::ChannelType::y: return dsModel::OutputType::lightCieY;
    case dss::ChannelType::shadePositionOutside: return dsModel::OutputType::shadePositionOutside;
    case dss::ChannelType::shadePositionIndoor: return dsModel::OutputType::shadePositionIndoor;
    case dss::ChannelType::shadeOpeningAngleOutside: return dsModel::OutputType::shadeOpeningAngleOutside;
    case dss::ChannelType::shadeOpeningAngleIndoor: return dsModel::OutputType::shadeOpeningAngleIndoor;
    case dss::ChannelType::transparency: return dsModel::OutputType::shadeTransparency;
    case dss::ChannelType::airFlowIntensity: return dsModel::OutputType::airFlowIntensity;
    case dss::ChannelType::airFlowDirection: return dsModel::OutputType::airFlowDirection;
    case dss::ChannelType::airFlapPosition: return dsModel::OutputType::airFlapOpeningAngle;
    case dss::ChannelType::airLouverPosition: return dsModel::OutputType::ventilationLouverPosition;
    case dss::ChannelType::heatingPower: return dsModel::OutputType::heatingPower;
    case dss::ChannelType::coolingCapacity: return dsModel::OutputType::coolingCapacity;
    case dss::ChannelType::audioVolume: return dsModel::OutputType::audioVolume;
    case dss::ChannelType::powerState: return dsModel::OutputType::powerState;
    case dss::ChannelType::airLouverAuto: return dsModel::OutputType::ventilationSwingMode;
    case dss::ChannelType::airFlowAuto: return dsModel::OutputType::ventilationAutoIntensity;
    case dss::ChannelType::waterTemperature: return dsModel::OutputType::waterTemperature;
    case dss::ChannelType::waterFlowRate: return dsModel::OutputType::waterFlowRate;
    case dss::ChannelType::powerLevel: return dsModel::OutputType::powerLevel;
    case dss::ChannelType::videoStation: return dsModel::OutputType::videoStation;
    case dss::ChannelType::videoInputSource: return dsModel::OutputType::videoInputSource;
  }
  return {};
}

std::optional<dsModel::MeasurementUsage> DSToEnum<dsModel::MeasurementUsage, int, void>::tryTo(int x) {
  switch (x) {
    case 1: return dsModel::MeasurementUsage::zone;
    case 2: return dsModel::MeasurementUsage::outdoor;
    case 3: return dsModel::MeasurementUsage::settings;
    case 4: return dsModel::MeasurementUsage::device;
    case 5: return dsModel::MeasurementUsage::deviceLastRun;
    case 6: return dsModel::MeasurementUsage::deviceAverage;
  }
  return {};
}

std::optional<dsModel::MeasurementType> DSToEnum<dsModel::MeasurementType, dss::SensorType, void>::tryTo(
    dss::SensorType x) {
  switch (x) {
    case dss::SensorType::ActivePower: return {};
    case dss::SensorType::OutputCurrent: return {};
    case dss::SensorType::ElectricMeter: return {};
    case dss::SensorType::TemperatureIndoors: return dsModel::MeasurementType::temperature;  // indoors in usage
    case dss::SensorType::TemperatureOutdoors: return dsModel::MeasurementType::temperature; // outdoors in usage
    case dss::SensorType::BrightnessIndoors: return dsModel::MeasurementType::brightness;    // indoors in usage
    case dss::SensorType::BrightnessOutdoors: return dsModel::MeasurementType::brightness;   // outdoors in usage
    case dss::SensorType::HumidityIndoors: return dsModel::MeasurementType::humidity;        // indoors in usage
    case dss::SensorType::HumidityOutdoors: return dsModel::MeasurementType::humidity;       // outdoors in usage
    case dss::SensorType::AirPressure: return {};
    case dss::SensorType::GustSpeed: return {};
    case dss::SensorType::GustDirection: return {}; // deprecated, not very meaningful
    case dss::SensorType::WindSpeed: return {};
    case dss::SensorType::WindDirection: return {};
    case dss::SensorType::Precipitation: return {};
    case dss::SensorType::CO2Concentration: return dsModel::MeasurementType::carbonDioxide;
    case dss::SensorType::COConcentration: return {};
    case dss::SensorType::SoundPressureLevel: return {};
    case dss::SensorType::RoomTemperatureSetpoint: return dsModel::MeasurementType::temperatureSetpoint;
    case dss::SensorType::RoomTemperatureControlVariable: return dsModel::MeasurementType::temperatureControlValue;
    case dss::SensorType::Status: return {};
    case dss::SensorType::Reserved1: return {};
    case dss::SensorType::Reserved2: return {};
    case dss::SensorType::OutputCurrent16A: return {};
    case dss::SensorType::ActivePowerVA: return {};
    case dss::SensorType::Temperature: return dsModel::MeasurementType::temperature;
    case dss::SensorType::Brightness: return dsModel::MeasurementType::brightness;
    case dss::SensorType::Humidity: return dsModel::MeasurementType::humidity;
    case dss::SensorType::WaterQuantity: return {};
    case dss::SensorType::WaterFlowRate: return {};
    case dss::SensorType::SunAzimuth: return {};
    case dss::SensorType::SunElevation: return {};
    case dss::SensorType::NotUsed: return {};
    case dss::SensorType::UnknownType: return {};
  }
  return {};
}

namespace dsApartment {} // namespace dsApartment
