/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "rest.h"

#include <ds/asio/test.h>
#include <ds/catch/catch.h>

#include "rest-mock.h"

using namespace trompeloeil;
using namespace std::literals;

namespace {

static const char* TAGS = "[dsApiRest][dsApi]";

SCENARIO("dsApiAsyncCallee", TAGS) {
  auto loop = ds::asio::TestLoopResource{};
  auto asyncCallee = dsApi::AsyncCalleeMock{};

  SECTION("blockingInvoke calls asyncInvoke") {
    const auto request = dsApi::HttpRequest{.path = "/a"};
    const auto result = dsApi::HttpResult{.status = 204};
    auto promise = std::unique_ptr<ds::Promise<dsApi::HttpResult>>{};
    REQUIRE_CALL(asyncCallee, asyncInvoke(_, _))
        .LR_SIDE_EFFECT(CHECK(_1 == request); promise = std::move(_2); loop.stop());
    auto thread = std::thread{[&] {
      CHECK(asyncCallee.blockingInvoke(loop.context, dsApi::HttpRequest{request}) == result);
      loop.stop();
    }};
    loop.run();
    promise->value = result;
    promise.reset();
    loop.run();
    thread.join();
  }
}

} // namespace
