/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds-api/event-fwd.h>
#include <ds-api/event.h>
#include <ds-api/rest.h>
#include <ds-net/buffered-message-writer.h>
#include <ds-net/message-stream.h>
#include <ds/asio/ds-signal.h>
#include <ds/asio/resource-fwd.h>
#include <ds/conf.h>

#include <boost/noncopyable.hpp>
#include <string>

#include "connector-fwd.h"
#include "protocol-fwd.h"

namespace dsBackend {

class ConnectorHook : public dsApi::AsyncCallee {
public:
  using Callback = std::function<void(std::variant<ds::JsonFragment, std::exception> &&)>;
  virtual ~ConnectorHook() = default;

  virtual Authenticate authenticate() = 0;

  virtual void permanentlyDisconnect() = 0;

  // from ds::AsyncCallee
  void asyncInvoke(dsApi::HttpRequest &&, std::unique_ptr<ds::Promise<dsApi::HttpResult>> &&) override = 0;
};

struct ConnectorSession {
  virtual ~ConnectorSession() = default;
  virtual dsNet::BufferedMessageWriter &bufferedWriter() = 0;
};

class Connector : public dsApi::AsyncCallee {
public:
  virtual ~Connector() = default;

  // thread safe
  virtual bool isConnected() const = 0;

  // thread safe
  virtual void setActive(bool) = 0;

  ds::Slots<bool> &isConnectedChanged() { return m_isConnectedChanged; }

  // from ds::AsyncCallee
  void asyncInvoke(dsApi::HttpRequest &&, std::unique_ptr<ds::Promise<dsApi::HttpResult>> &&) override = 0;

  // client websocket session
  virtual ConnectorSession *maybeSession() = 0;
  virtual std::string lastDisconnectReason() const = 0;

  virtual dsApi::EventSink &eventSink() = 0;

protected:
  ds::Signal<bool> m_isConnectedChanged;
};

///
/// Maintains stream client connection and provides bus interface for it.
///
std::unique_ptr<Connector> makeConnector(ds::asio::ResourceBase &parent, std::string &&id, dsNet::MessageStreamClient &,
                                         ConnectorHook &, dsApi::EventHub &);

} // namespace dsBackend
