// Generated file.  DO NOT EDIT!
// source: installation.yaml

// clang-format off

#pragma once


/// Installation
///
/// Installation, floors, dsMeters

#include "installation-fwd.h"
#include <ds/enum.h>
#include <ds/clock.h>

#include "common-fwd.h"
#include "devices-fwd.h"
#include "groups-fwd.h"
#include "common.h"
#include "model-fwd.h"

#include <ds/json.h>


/// Deployment environment
///
/// Example: production
DS_REFLECTED_ENUM((dsModel), Environment,
    (none)
    (development)
    (alpha)
    (beta)
    (production)
);


/// Authentication provider. Closed list of integrated providers.
DS_REFLECTED_ENUM((dsModel), AuthorizeControllerArgsProvider,
    (none)
    (codeGrantFlowSimulator)
    (implicitFlowSimulator)
    (siemens)
    (siemensSimulator)
    (logitech)
    (netatmo)
    (netatmo2)
    (ikettle)
    (bosch)
);

DS_REFLECTED_ENUM((dsModel), ParameterCacheStatus,
    (none)
    (updating)
    (incomplete)
    (complete)
);


/// Speed of the cache update. "slow" is more compatible with ongoing
DS_REFLECTED_ENUM((dsModel), UpdateParameterCacheArgsMode,
    (none)
    (slow)
    (fast)
);

namespace dsModel {
struct FloorAttributes {

  /// User given name
  ///
  /// Example: Basement
  std::string name{};

  /// Order id used for sorting
  ///
  /// Example: 20
  std::optional<double> orderId{};
  std::vector<ZoneId> zones{};

  ~FloorAttributes();
};

struct Floor {
  FloorId id{};
  std::string type = "floor";
  ds::DateTime lastChanged{};
  FloorAttributes attributes{};

  ~Floor();
};

struct Floors {
  std::vector<Floor> floors{};

  ~Floors();
};

struct Coordinates {

  /// geographic coordinate in decimal degrees
  std::optional<double> longitude{};

  /// geographic coordinate in decimal degrees
  std::optional<double> latitude{};

  ~Coordinates();
};

struct InstallationAttributes {
  Coordinates location{};

  /// ISO 3166-1 alpha-2; country reserved for full country name
  ///
  /// Example: DE
  std::string countryCode{};

  /// Human readable city name
  ///
  /// Example: Dresden
  std::string city{};

  /// Time zone name. See `TZ database name` in https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
  ///
  /// Example: Europe/Berlin
  std::string timeZone{};

  ~InstallationAttributes();
};

struct Installation {
  InstallationId id{};
  std::string type = "installation";
  ds::DateTime lastChanged{};
  InstallationAttributes attributes{};

  ~Installation();
};

struct ApplicationTokenAttributes {

  /// Name of the application
  ///
  /// Example: Smartphone
  std::string name{};

  ~ApplicationTokenAttributes();
};

struct ApplicationToken {
  ApplicationTokenId id{};
  std::string type = "applicationToken";
  ApplicationTokenAttributes attributes{};

  ~ApplicationToken();
};

struct ApplicationTokens {
  std::vector<ApplicationToken> applicationTokens{};

  ~ApplicationTokens();
};

struct DSServerAttributes {

  /// User given host name without any hostname related mangling
  ///
  /// Example: dSS
  std::string name{};

  /// dSS11-E, dSSIP, dSS20, dSS22, dSS11-1GB, ...
  ///
  /// Example: dSS20
  std::string model{};
  std::string gtin{};
  std::string dsid{};

  /// Example: 1.16.0-beta04
  std::string version{};
  Environment environment{};

  ~DSServerAttributes();
};

struct DSServer {
  DSServerId id{};
  std::string type = "dsServer";
  ds::DateTime lastChanged{};
  DSServerAttributes attributes{};

  ~DSServer();
};


/// Authorize controller arguments.
struct AuthorizeControllerArgs {

  /// Provider specific authentication scope
  ///
  /// Example: control
  std::string authScope{};

  /// Provider specifice authentication scope
  ///
  /// Example: code
  std::string authResponseType{};

  /// Landing page after authentication has finished
  ///
  /// Example: https://dss.local/netatmo
  std::string redirectUri{};

  /// Authentication provider. Closed list of integrated providers.
  AuthorizeControllerArgsProvider provider{};
  ControllerId controller{};
  Language lang{};

  ~AuthorizeControllerArgs();
};

struct AuthorizeControllerResult {

  /// Indicates the URL to redirect to for authorization. At the end of the authorization, access credentials are stored inside controller and client is redirected back to `redirectUri`.
  /// Like `Location` response header with 30x status response. But some browsers don't allow to access `Location` header in certains situations.
  std::string location{};

  ~AuthorizeControllerResult();
};

struct MaintenanceAttributes {
  bool enabled{};
  std::string privateKey{};
};

struct MaintenanceDataData {
  MaintenanceAttributes attributes{};
  std::string type{};
  std::string id{};
};

struct MaintenanceData {
  MaintenanceDataData data{};
};

struct StoreControllerAccessTokenArgs {
  std::string authData{};
  std::string authScope{};

  /// Arbitrary json params
  ds::JsonFragment params{};

  ~StoreControllerAccessTokenArgs();
};

struct ControllerAttributes {

  /// User given name
  ///
  /// Example: DSM20
  std::string name{};
  std::string technicalName{};

  ~ControllerAttributes();
};

struct Controller {
  ControllerId id{};
  std::string type = "controller";
  ds::DateTime lastChanged{};
  ControllerAttributes attributes{};

  ~Controller();
};

struct Controllers {
  std::vector<Controller> controllers{};

  ~Controllers();
};

struct ParameterCacheFunctionBlockAttributes {

  /// in %
  std::optional<double> completion{};

  ~ParameterCacheFunctionBlockAttributes();
};

struct ParameterCacheFunctionBlock {
  FunctionBlockId id{};
  std::string type = "parameterCacheFunctionBlock";
  ds::DateTime lastChanged{};
  ParameterCacheFunctionBlockAttributes attributes{};

  ~ParameterCacheFunctionBlock();
};

struct UpdateParameterCacheArgs {

  /// Speed of the cache update. "slow" is more compatible with ongoing
  UpdateParameterCacheArgsMode mode{};

  ~UpdateParameterCacheArgs();
};

struct ParameterCacheAttributes {
  ParameterCacheStatus status{};
  std::optional<int> total{};
  std::optional<int> completed{};
  std::optional<double> completion{};
  ds::Clock::duration remainingTime{};

  ~ParameterCacheAttributes();
};

struct ParameterCacheIncluded {
  std::vector<ParameterCacheFunctionBlock> functionBlocks{};

  ~ParameterCacheIncluded();
};


/// Device parameter cache database provides fast access to terminal block parameters and scene configuration.
/// Certain system functionality, like the empty click prevention, depends on this data and requires that the
/// update process has run to completion.
struct ParameterCache {
  ParameterCacheId id{};
  std::string type = "parameterCache";
  ds::DateTime lastChanged{};
  ParameterCacheAttributes attributes{};
  ParameterCacheIncluded included{};

  ~ParameterCache();
};

} // namespace dsModel

DS_JSON_DECLARE_STRUCT((dsModel), FloorAttributes,
    (name)
    (orderId)
    (zones)
);
DS_JSON_DECLARE_STRUCT((dsModel), Floor,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), Floors,
    (floors)
);
DS_JSON_DECLARE_STRUCT((dsModel), Coordinates,
    (longitude)
    (latitude)
);
DS_JSON_DECLARE_STRUCT((dsModel), InstallationAttributes,
    (location)
    (countryCode)
    (city)
    (timeZone)
);
DS_JSON_DECLARE_STRUCT((dsModel), Installation,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), ApplicationTokenAttributes,
    (name)
);
DS_JSON_DECLARE_STRUCT((dsModel), ApplicationToken,
    (id)
    (type)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), ApplicationTokens,
    (applicationTokens)
);
DS_JSON_DECLARE_STRUCT((dsModel), DSServerAttributes,
    (name)
    (model)
    (gtin)
    (dsid)
    (version)
    (environment)
);
DS_JSON_DECLARE_STRUCT((dsModel), DSServer,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), AuthorizeControllerArgs,
    (authScope)
    (authResponseType)
    (redirectUri)
    (provider)
    (controller)
    (lang)
);
DS_JSON_DECLARE_STRUCT((dsModel), MaintenanceAttributes,
    (enabled)
    (privateKey)
);
DS_JSON_DECLARE_STRUCT((dsModel), MaintenanceDataData,
    (attributes)
    (type)
    (id)
);
DS_JSON_DECLARE_STRUCT((dsModel), MaintenanceData,
    (data)
);
DS_JSON_DECLARE_STRUCT((dsModel), AuthorizeControllerResult,
    (location)
);
DS_JSON_DECLARE_STRUCT((dsModel), StoreControllerAccessTokenArgs,
    (authData)
    (authScope)
    (params)
);
DS_JSON_DECLARE_STRUCT((dsModel), ControllerAttributes,
    (name)
    (technicalName)
);
DS_JSON_DECLARE_STRUCT((dsModel), Controller,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), Controllers,
    (controllers)
);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheFunctionBlockAttributes,
    (completion)
);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheFunctionBlock,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), UpdateParameterCacheArgs,
    (mode)
);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheAttributes,
    (status)
    (total)
    (completed)
    (completion)
    (remainingTime)
);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheIncluded,
    (functionBlocks)
);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCache,
    (id)
    (type)
    (lastChanged)
    (attributes)
    (included)
);

