/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "scenarios.h"

#include <ds-api/service.h>
#include <ds/catch/catch.h>
#include <ds/log.h>

using namespace trompeloeil;

namespace {

static const char *TAGS = "[dsModelScenarios][dsModel]";

SCENARIO("Scenario json serialization", TAGS) {
  // variants are somewhat tricky to serialize
  CHECK(ds::toJson(dsModel::Scenario{}) == R"(null)");
  CHECK(ds::toJson(dsModel::Scenario{dsModel::ApplicationZoneScenario{.id = dsModel::ApplicationZoneScenarioId{"id0"},
                                                                      .type = "applicationZoneScenario",
                                                                      .attributes = {.name = "name0"}}}) ==
        R"({"id":"id0","type":"applicationZoneScenario","attributes":{"name":"name0"}})");
  CHECK(ds::toJson(dsModel::Scenario{
            dsModel::UserDefinedAction{.id = dsModel::UserDefinedActionId{"id0"},
                                       .type = "userDefinedAction",
                                       .attributes = dsModel::UserDefinedActionAttributes{.name = "name0"}}}) ==
        R"({"id":"id0","type":"userDefinedAction","attributes":{"name":"name0"}})");
  CHECK(
      ds::toJson(dsModel::Scenario{
          dsModel::DeviceScenario{.id = dsModel::DeviceScenarioId{"id0"},
                                  .type = "deviceScenario",
                                  .attributes = {.name = {.domain = {}, .message = "name0"},
                                                 .dsDevice = dsModel::DSDeviceId{"dev0"},
                                                 .actionId = dsModel::DeviceScenarioActionId{"action0"}}}}) ==
      R"({"id":"id0","type":"deviceScenario","attributes":{"name":{"message":"name0"},"dsDevice":"dev0","actionId":"action0"}})");
}

} // namespace
