/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds-net/message-stream.h>
#include <ds/asio/resource.h>

#include "buffered-message-writer.h"
#include "session.h"

namespace dsNet {

struct SignalrSessionHooks {
  std::function<void()> opened = [] {};
  std::function<void(error_code)> closed = [](auto&&) {};
  std::function<void(std::string&& target, std::vector<ds::JsonFragment>&&)> invokeNoReply = [](auto&&, auto&&) {};
  BufferedMessageWriterConfig bufferedMessageWriterConfig{};
};

struct SignalrSession : virtual ISession {
  virtual void asyncRun(SignalrSessionHooks&&) = 0;
  virtual void invokeNoReply(std::string&& target, std::vector<ds::JsonFragment>&& arguments) = 0;
};

ClosePtr<SignalrSession> makeSignalrSession(Sessions& parent, ClosePtr<dsNet::MessageStream>&&);

struct SignalrAcceptorHooks {
  /// Message stream was accepted and needs to be processed.
  std::function<void(ClosePtr<SignalrSession>)> accept;
};

/// WSAcceptor accepts `MessageStream`s  and provides `SignalrSession`s
struct SignalrAcceptor {
  virtual ~SignalrAcceptor() = default;
  virtual void asyncRun(SignalrAcceptorHooks&&) = 0;
};

std::unique_ptr<SignalrAcceptor> makeSignalrAcceptor(ds::asio::ResourceBase& parent,
                                                     std::unique_ptr<MessageStreamAcceptor>);

} // namespace dsNet
