/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds-net/message-stream.h>
#include <ds/asio/resource.h>
#include <ds/reflect.h>
#include <ds/str.h>

#include <boost/asio/ssl/context.hpp>
#include <boost/beast/http/message.hpp>
#include <boost/beast/http/string_body.hpp>

#include "tcp-acceptor.h"

namespace dsNet {

/// Parsed websocket url (ssl only for now).
struct WSSUrl {
  std::string host;
  std::string port{"443"};
  std::string target;
};

/// Parses string like wws://host:port/target into WWSUrl
WSSUrl parseWSSUrl(gsl::cstring_span);

/// Client ssl context verifying server certificate against certificates trusted by system
boost::asio::ssl::context systemClientSslContext();

struct WSSClientConfig {
  std::string url;
};

std::unique_ptr<MessageStreamClient> makeWSSClient(ds::asio::ResourceBase &, WSSClientConfig &&,
                                                   boost::asio::ssl::context &);

namespace beast = boost::beast;
namespace http = boost::beast::http;
struct WebsocketAcceptorHooks {
  std::function<void(const http::request<http::string_body> &)> authenticate = [](auto &&) {};
};

std::unique_ptr<MessageStreamAcceptor> makeWSAcceptor(ds::asio::ResourceBase &parent, std::unique_ptr<TcpAcceptor>,
                                                      WebsocketAcceptorHooks && = {});

std::unique_ptr<MessageStreamAcceptor> makeWSSAcceptor(ds::asio::ResourceBase &parent, std::unique_ptr<TcpAcceptor>,
                                                       boost::asio::ssl::context &, WebsocketAcceptorHooks && = {});

} // namespace dsNet

DS_JSON_DECLARE_STRUCT((dsNet), WSSUrl, (host)(port)(target));
DS_JSON_DECLARE_STRUCT((dsNet), WSSClientConfig, (url));
