/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "back-off.h"
#include <ds/asio/test.h>

namespace {

static const char* TAGS = "[dsAsioBackOff][dsAsio][ds]";
using namespace std::chrono_literals;

SCENARIO("dsAsioBackOff", TAGS) {
  auto loop = ds::asio::TestLoopResource{};
  auto config = ds::asio::BackOffConfig{};
  auto backOff = ds::asio::BackOff(loop, "backOff", ds::asio::BackOffConfig{config});

  THEN("next() is 0") {
    CHECK(backOff.next() == ds::Clock::duration{});

    THEN("next() is config.first") {
      auto peekedX = backOff.peek();
      auto x = backOff.next();
      CHECK(x == peekedX);
      CHECK(x == config.first);

      THEN("next() is increasing by rughly by factor 2 until max") {
        while (x < config.max) {
          auto oldX = x;
          peekedX = backOff.peek();
          x = backOff.next();
          INFO(oldX);
          INFO(x);
          CHECK(x == peekedX);
          CHECK(x >= std::min(oldX * 2, config.max));
          CHECK(x <= oldX * 3);
        }

        THEN("next() is max") { CHECK(backOff.next() == config.max); }
        WHEN("reset()") {
          backOff.reset();
          THEN("next() is 0") { CHECK(backOff.next() == ds::Clock::duration{}); }
        }
      }
    }
  }
}
} // namespace
