/*
    Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ds/clock.h>
#include <ds/reflect.h>
#include <boost/asio/io_service.hpp>
#include <functional>
#include <memory>

namespace ds::asio {

struct LatencyWatchdogConfig {
  ds::Clock::duration warningLatency{};
  ds::Clock::duration assertLatency{};
};

struct LatencyWatchdogHooks {
  std::function<void(ds::Clock::duration)> fire = [](ds::Clock::duration) {};
  std::function<void()> pinged = []() {};
  std::function<void()> ponged = []() {};
};

/// Software watchdog that guards that io is not stalled.
class LatencyWatchdog {
public:
  /// Regurarly pings `watchedIo` from `watchdogIo`,
  /// call `fire` on `latency` failure.
  LatencyWatchdog(boost::asio::io_service &io, boost::asio::io_service &watchdogIo, ds::Clock::duration latency,
                  LatencyWatchdogHooks &&hooks);

  /// Regurarly pings `watchedIo` from internal thread,
  /// call `fire` on `latency` failure.
  LatencyWatchdog(boost::asio::io_service &io, ds::Clock::duration latency, LatencyWatchdogHooks &&hooks);

  /// Regurarly pings `watchedIo` from internal thread,
  /// log warning / assert on latency failures specified by `config`.
  LatencyWatchdog(boost::asio::io_service &watchedIo, LatencyWatchdogConfig config);

  ~LatencyWatchdog();

private:
  struct Impl;
  const std::unique_ptr<Impl> m_impl;
};

} // namespace ds::asio

DS_REFLECT_STRUCT((ds)(asio), LatencyWatchdogConfig, (warningLatency)(assertLatency));
