#include "promise.h"

#include <ds/asio/test.h>
#include <ds/catch/catch.h>
#include <ds/clock-mock.h>

#include <trompeloeil.hpp>

static const char *TAGS = "[dsPromise][ds]";

using namespace trompeloeil;

namespace {

struct BoolFulfilled {
public:
  MAKE_MOCK1(call, void(std::optional<bool>));
};

TEST_CASE("dsPromise", TAGS) {
  ds::ClockMocker clock;
  ds::asio::TestLoopResource loop;
  GIVEN("bool promise") {
    auto boolFulfilled = BoolFulfilled{};
    auto promise = ds::makePromise<bool>(loop.io, [&](std::optional<bool> &&x) { boolFulfilled.call(std::move(x)); });

    THEN("fulfilled with nulopt on promise reset") {
      REQUIRE_CALL(boolFulfilled, call(std::nullopt)).LR_SIDE_EFFECT(loop.stop());
      promise.reset();
      loop.run();
      SUCCEED();
    }

    THEN("fulfilled with value on promise reset") {
      promise->value = true;
      REQUIRE_CALL(boolFulfilled, call(true)).LR_SIDE_EFFECT(loop.stop());
      promise.reset();
      loop.run();
      SUCCEED();
    }
  }
}

} // namespace
