/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "conf.h"

#include <ds/catch/catch.h>
#include <ds/json.h>
#include <ds/log.h>

#include "system-mock.h"

static const char* TAGS = "[dsConf][ds]";

namespace {
struct X {
  int x = 1;
  int y = 2;
};
} // namespace

DS_JSON_ANONYMOUS_STRUCT(X, (x)(y));

namespace {
TEST_CASE("dsConf", TAGS) {
  auto system = ds::SystemMock();
  auto conf = ds::makeConf(system);
  SECTION("readOr reads etc") {
    REQUIRE_CALL(system, readFileIfExists("/etc/ds-conf/test1.json")).RETURN("1");
    CHECK(conf->readOr("test1", 7) == 1);
  }

  SECTION("readOr returns default value without etc") { CHECK(conf->readOr<int>("testNoMatch", 7) == 7); }

  SECTION("check partial config of struct") {
    REQUIRE_CALL(system, readFileIfExists("/etc/ds-conf/partialConfig.json")).RETURN(R"({"x":7})");
    CHECK(conf->readOr("partialConfig", X{}) == X{.x = 7, .y = 2});
  }

  SECTION("default value for struct if no file in etc") {
    CHECK(conf->readOr("partialConfig", X{}) == X{.x = 1, .y = 2});
  }
}

} // namespace
