/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "latency-statistics.h"
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/count.hpp>
#include <boost/accumulators/statistics/max.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/min.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include "iostream"

using namespace boost::accumulators;

namespace ds {

struct LatencyStatistics::Impl {
  accumulator_set<double, stats<tag::count, tag::min, tag::mean, tag::max, tag::variance(lazy)>> statistics;
};

LatencyStatistics::LatencyStatistics() : m_impl(std::make_unique<Impl>()) {}
LatencyStatistics::~LatencyStatistics() = default;

void LatencyStatistics::operator()(double x) {
  m_impl->statistics(x);
}

int LatencyStatistics::count() const {
  return boost::accumulators::count(m_impl->statistics);
}

double LatencyStatistics::min() const {
  return boost::accumulators::min(m_impl->statistics);
}

double LatencyStatistics::avg() const {
  return boost::accumulators::mean(m_impl->statistics);
}

double LatencyStatistics::max() const {
  return boost::accumulators::max(m_impl->statistics);
}

double LatencyStatistics::sdev() const {
  return sqrt(boost::accumulators::variance(m_impl->statistics));
}

void LatencyStatistics::drop() {
  m_impl = std::make_unique<Impl>();
}

std::ostream &operator<<(std::ostream &ostream, const LatencyStatistics &x) {
  return ostream << "min:" << x.min() << " avg:" << x.avg() << " max:" << x.max() << " sdev:" << x.sdev()
                 << " count:" << x.count();
}

} // namespace ds
