#include "backend.h"
#include <ds/log.h>
#include <boost/locale/localization_backend.hpp>
#include <boost/locale/message.hpp>
#include <mutex>
#include "messages.h"

/// boost::locale forces us to use std::auto_ptr.
/// There is nothing we can do to fix the warning.
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

namespace ds {
namespace locale {

namespace {

class MessagesBackend : public boost::locale::localization_backend {
public:
  MessagesBackend() {}
  MessagesBackend(MessagesBackend const &other) : boost::locale::localization_backend(), m_info(other.m_info) {}
  virtual MessagesBackend *clone() const { return new MessagesBackend(*this); }

  void set_option(std::string const &name, std::string const &value) {
    if (name == "locale")
      m_info.language = value;
    else if (name == "message_path")
      m_info.paths.push_back(value);
    else if (name == "message_application")
      m_info.domains.push_back(value);
  }
  void clear_options() {
    m_info.language.clear();
    m_info.paths.clear();
    m_info.domains.clear();
  }

  virtual std::locale install(
      std::locale const &base, boost::locale::locale_category_type category, boost::locale::character_facet_type type) {
    switch (category) {
      case boost::locale::message_facet:
        return std::locale(base, makeMessages(MessagesInfo(m_info)));
    }
    DS_FAIL_REQUIRE("Unrecognized category", category);
  }

private:
  MessagesInfo m_info;
};

// prevent initialization order fiasco
std::mutex &mutex() {
  static std::mutex x;
  return x;
}
boost::locale::localization_backend_manager &lockedBackend() {
  static auto x = [] {
    auto manager = boost::locale::localization_backend_manager::global();
    manager.select("std");
    manager.add_backend("dsMessages", std::auto_ptr<boost::locale::localization_backend>(new MessagesBackend()));
    manager.select("dsMessages", boost::locale::message_facet);
    return manager;
  }();
  return x;
}

} // namespace

const boost::locale::localization_backend_manager &backend() {
  std::unique_lock<std::mutex> lock(mutex());
  return lockedBackend();
}

} // locale
} // ds
