/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include "businterface.h"
#include "dsbusinterfaceobj.h"

namespace dss {

class DSActionRequest : public DSBusInterfaceObj, public ActionRequestInterface {
public:
  DSActionRequest() : DSBusInterfaceObj(), m_pBusEventSink(NULL) {}

  void callScene(AddressableModelItem *pTarget, const uint16_t _scene, Caller &&caller, const bool _force) override;
  void callSceneMin(AddressableModelItem *pTarget, const uint16_t _scene, Caller &&caller) override;
  void saveScene(AddressableModelItem *pTarget, const uint16_t _scene, Caller &&caller) override;
  void undoScene(AddressableModelItem *pTarget, const uint16_t _scene, Caller &&caller) override;
  void undoSceneLast(AddressableModelItem *pTarget, Caller &&caller) override;
  void blink(AddressableModelItem *pTarget, Caller &&caller) override;
  void setValue(AddressableModelItem *pTarget, const uint8_t value, Caller &&caller) override;
  void increaseOutputChannelValue(AddressableModelItem *pTarget, ChannelType channel, Caller &&caller) override;
  void decreaseOutputChannelValue(AddressableModelItem *pTarget, ChannelType channel, Caller &&caller) override;
  void stopOutputChannelValue(AddressableModelItem *pTarget, ChannelType channel, Caller &&caller) override;
  void pushSensor(AddressableModelItem *pTarget, dsuid_t _sourceID, SensorType _sensorType, double _sensorValueFloat,
                  Caller &&caller) override;
  void muteFire() override;
  void setState(Group *group, const uint8_t zoneState, const uint8_t areaState1, const uint8_t areaState2,
                const uint8_t areaState3, const uint8_t areaState4) override;
  void setState(Cluster *cluster, const uint8_t state) override;
  void stopStopTimer(Group *group, const uint8_t zoneOrArea) override;
  void stopStopTimer(Cluster *cluster) override;

  void setBusEventSink(BusEventSink *eventSink);

  //< @ret true if locked
  bool isOperationLock(const dsuid_t &dSM, int clusterId) override;

private:
  BusEventSink *m_pBusEventSink;
}; // DSActionRequest

} // namespace dss
