/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "format-json.h"

#include <ds/catch/catch.h>
#include <ds/json.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("dssJsonFormatTest");
static const char* TAGS = "[dss][dssJsonFormat]";

using namespace std::literals;
using namespace trompeloeil;
using namespace dss;

namespace {

struct S {
  int one;
  std::string two;
};

struct T {
  S alpha;
  std::vector<S> beta;
};

SCENARIO("JsonFormat", TAGS) {
  SECTION("insert newlines") {
    T t{.alpha{1, "foo"}};
    for (const auto& line : formatJson(ds::toJson(t))) {
      DS_NOTICE(line);
    }
    CHECK(formatJson(ds::toJson(t)) ==
          std::vector<std::string>{"{"s, R"(  "alpha":{)", R"(    "one":1,"two":"foo")", "  }"s, "}"s});
  }
}

} // namespace

DS_JSON_ANONYMOUS_STRUCT(S, (one)(two));
DS_JSON_ANONYMOUS_STRUCT(T, (alpha)(beta));
