/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "format-json.h"

#include <ds/log.h>

static ds::LogChannel dsLogChannel("dssFormatJson");

using namespace std::literals;

namespace dss {

namespace {
std::vector<std::string_view> indents{
    ""sv,           "  "sv,           "    "sv,           "      "sv,           "        "sv,
    "          "sv, "            "sv, "              "sv, "                "sv,
};
}

std::vector<std::string> formatJson(const std::string& blurp) {
  std::string::size_type start, end = 0;
  DS_DEBUG(blurp);
  std::vector<std::string> result{};

  int indent = 0;
  auto nextDelim = [&blurp](auto cur) { return blurp.find_first_of("[{}]", cur); };
  for (auto cur = nextDelim(0); cur != std::string::npos; cur = nextDelim(++cur)) {
    switch (blurp.at(cur)) {
      case '{':
      case '[':
        start = std::exchange(end, cur + 1); // include opening bracket/brace
        result.push_back(std::string(indents[indent]) + blurp.substr(start, end - start));
        indent = std::min(indent + 1, gsl::narrow_cast<int>(indents.size()) - 1);
        DS_ASSERT(indent < gsl::narrow_cast<int>(indents.size()));
        break;
      case '}':
      case ']':
        start = std::exchange(end, cur); // suppress closing bracket/brace
        result.push_back(std::string(indents[indent]) + blurp.substr(start, end - start));
        indent = std::max(indent - 1, 0);
        break;
    }
  }
  if (auto trailing = blurp.substr(end); !trailing.empty()) {
    result.push_back(std::string(indents[indent]) + trailing); // closing bracket
  }
  return result;
}

} // namespace dss
