/*
    Copyright (c) 2009,2010,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <digitalSTROM/dsuid.h>
#include <ds/system.h>

#include "ds485types.h"
#include "metering/archives-fwd.h"
#include "metering/metering-fwd.h"

namespace dss::metering::archives {

using ModbusMeteringArchiveId = std::pair<std::string, Category>;

class ModbusMeteringArchives {
public:
  virtual ~ModbusMeteringArchives() = default;

  virtual void create(const ModbusMeteringArchiveId& archiveId) = 0;
  virtual void addValue(const ModbusMeteringArchiveId& archiveId, ArchiveValue&& value) = 0;

  virtual std::optional<ArchiveValue> lastValue(const ModbusMeteringArchiveId& archiveId) const = 0;
  virtual ValueRange xport(time_t start, time_t end, int step, std::vector<ModbusMeteringArchiveId> archiveIds, SeriesTypes type,
                           bool energyInWh) const = 0;

  virtual std::optional<ds::DateTime> lastTimestamp(ModbusMeteringArchiveId archiveId) const = 0;
  virtual void flush(std::vector<ModbusMeteringArchiveId> archiveIds) const = 0;

  // unit tests
  virtual bool validateDBSeries(const ModbusMeteringArchiveId& archiveId, bool& _tunePowerMaxSetting) = 0; // TODO const
  virtual void tuneDBPowerSettings(const ModbusMeteringArchiveId& archiveId) = 0;
};

std::unique_ptr<ModbusMeteringArchives> makeModbusArchive(ArchivesConnection& archivesConnection,
                                                          ArchivesStorage& archivesStorage, ds::System& system);

} // namespace dss::metering::archives
