/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ds/date-time.h>
#include <ds/system-fwd.h>

#include "metering/archives-fwd.h"

namespace dss::metering::archives {

enum class ValidationResult { ok, needsMigration, failure };

class OpenMeteringSystemArchive {
public:
  virtual ~OpenMeteringSystemArchive() = default;

  virtual void create(const OpenMeteringSystemId& openMeteringSystemId,
                      ds::DateTime startTime = std::chrono::system_clock::now()) = 0;
  virtual void addValue(const OpenMeteringSystemId& openMeteringSystemId, readings::OpenMeteringSystem&& value) = 0;

  virtual std::optional<readings::OpenMeteringSystem> lastValue(
      const OpenMeteringSystemId& openMeteringSystemId) const = 0;
  virtual ValueRange getSeries(const OpenMeteringSystemId& openMeteringSystemId, Category category,
                               SeriesTypes seriesType, time_t start, time_t end, int step,
                               EnergyUnit energyUnit = EnergyUnit::Wh) const = 0;

  virtual ValidationResult validate(const OpenMeteringSystemId& openMeteringSystemId) const = 0;
  virtual void migrate(const OpenMeteringSystemId& openMeteringSystemId) = 0;

  // TODO, flush/lastTimestamp are common with controller, only differ in filename
  virtual std::optional<ds::DateTime> lastTimestamp(OpenMeteringSystemId openMeteringSystemId) const = 0;
  virtual void flush(std::vector<OpenMeteringSystemId> ids) const = 0;
};

std::unique_ptr<OpenMeteringSystemArchive> makeOpenMeteringSystemArchive(ArchivesConnection& archivesConnection,
                                                                         ArchivesStorage& archivesStorage,
                                                                         ds::System& system);

} // namespace dss::metering::archives
