/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "metering/config.h"

#include <ds/catch/catch.h>
#include <trompeloeil.hpp>

static ds::LogChannel dsLogChannel("dssMeteringConfigTest");

using namespace dss::metering;
using trompeloeil::_;

namespace {

static const char* TAGS = "[metering][config]";

SCENARIO("config", TAGS) {
  SECTION("check default") {
    auto config = ElectricityMeteringConfig{};
    CHECK(config.sensorValueTimeout == 3s);

    const auto& discontinuityDetection = config.discontinuityDetection;
    CHECK(discontinuityDetection.interval == 3s);
    CHECK(discontinuityDetection.maxProduction == 50'000);
    CHECK(discontinuityDetection.maxConsumption == 8'000);

    CHECK(config.socketPath == "/var/run/dss-metering.sock"s);
  }

  SECTION("sample json format") {
    CHECK(
        ds::fromJson<ElectricityMeteringConfig>(
            R"({"sensorValueTimeout":10.0,"discontinuityDetection":{"interval":7.0,"maxConsumption":1000,"maxProduction":2000}})") ==
        ElectricityMeteringConfig{10s, {7s, 1'000, 2'000}});
  }
  SECTION("maxInterpolationInterval") {
    auto config = ds::fromJson<ElectricityMeteringConfig>(R"({"maxInterpolationInterval":10.0})");
    CHECK(config.maxInterpolationInterval == 10s);
  }
}

} // namespace
