#pragma once

#include "ds-model/meterings-fwd.h"
#include "metering/archives-fwd.h"
#include "ds-model/meterings.h"
#include "metering.h"

namespace dss::metering {

struct SerialPortConfigFile {
  dsModel::SerialPortConfig serialPortConfig{};
};

struct UserMeterConfigFile {
  std::vector<dsModel::ElectricityMeterConfig> userMeterConfigs;
};

struct AttachedMetersConfigFile {
  std::vector<dsModel::ModbusMeter> attachedMeters;
};

class ModbusMeteringSystem {
public:
  using Reading = readings::ModbusMeteringSystem;

  virtual ~ModbusMeteringSystem() = default;

  virtual std::vector<std::tuple<ModbusMeteringId, uint8_t, bool, std::string, dsModel::MeteringType>> meterings() const = 0;

  virtual dsModel::SerialPortConfig getSerialPortConfig() const = 0;
  virtual void setSerialPortConfig(const dsModel::SerialPortConfig& serialPortConfig) = 0;

  virtual std::vector<dsModel::ModbusMeter> getModbusAttachedMeters() const = 0;

  virtual void addModbusAttachedMeter(const dsModel::ModbusMeter& meter) = 0;

  virtual void deleteModbusAttachedMeter(const dsModel::MeteringId& meter) = 0;

  virtual std::vector<dsModel::ElectricityMeterConfig> getModbusSupportedMeters() const = 0;

  virtual std::vector<dsModel::ElectricityMeterConfig> getModbusUserMeters() const = 0;

  virtual void addModbusSupportedMeter(const dsModel::ElectricityMeterConfig& meterConfig) = 0;

  virtual void deleteModbusSupportedMeter(const dsModel::MeteringConfigId& meterConfigId) = 0;

  virtual void addReading(Reading&& reading) = 0;

  virtual void unmarkGlobalAll() = 0;

  virtual std::optional<LastReading> getLastReading(ModbusMeteringId id, dsModel::MeteringType meteringType,
                                                    EnergyUnit energyUnit) const = 0;

  virtual ValueRange getSeries(std::vector<ModbusMeteringId> ids, dsModel::MeteringType meteringType, 
                               SeriesTypes type, std::chrono::seconds resolution,
                               const TimeDuration& timeDuration, int valueCount, EnergyUnit energyUnit) const = 0;

  virtual double get15MinuteAverage(std::vector<ModbusMeteringId> ids, dsModel::MeteringType meteringType,
                                    metering::SeriesTypes type, Unit unit) const = 0;
};

std::unique_ptr<ModbusMeteringSystem> makeModbusMeteringSystem(ArchivesConnection& archivesConnection,
                                                               ArchivesStorage& archivesStorage);

std::unique_ptr<ModbusMeteringSystem> makeModbusMeteringSystemMock(std::unique_ptr<archives::ModbusMeteringArchives> modbusMeteringArchives, 
                                                                  ArchivesStorage& archivesStorage);

} // namespace dss::metering

DS_JSON_DECLARE_STRUCT((dss::metering), SerialPortConfigFile, (serialPortConfig));
DS_JSON_DECLARE_STRUCT((dss::metering), UserMeterConfigFile, (userMeterConfigs));
DS_JSON_DECLARE_STRUCT((dss::metering), AttachedMetersConfigFile, (attachedMeters));