#pragma once

#include <boost/asio.hpp>
#include <chrono>
#include <string>

#include "socket-writer.h"
#include "http_client.h"
#include "ds-model/meterings.h"
#include "metering/readings/modbusmeteringsystem.h"

using Reading = dss::metering::readings::ModbusMeteringSystem;

namespace dss::modbusreader {


struct SenderQueue {
  SenderQueue(boost::asio::io_context& io, std::string socketPath) : m_io(io), m_socketPath(socketPath) {}

  void send(std::string);

  boost::asio::io_context& m_io;
  std::string m_socketPath;
};

struct SerialConnection {
  SerialConnection(boost::asio::io_context& io, SenderQueue& senderQueue) : m_io(io), m_senderQueue(senderQueue) {
    m_busy = false;
    m_intervalUSec = 20000;
  }

  void start();

  void doRead();

  void querySerialConnectionConfig();

  void queryAttachedMeters();

  void readMeter(int slaveAddress, dsModel::ElectricityMeterConfig);

  void doReconnect();

  bool isBusy() { return m_busy; }

  HttpClient m_client;
  dsModel::SerialPortConfig m_config;
  boost::asio::io_context& m_io;
  bool m_connected;
  SenderQueue& m_senderQueue;
  std::vector<dsModel::ModbusMeter> m_attachedMeters;
  std::unordered_map<int, Reading> m_previousReading;
  bool m_busy;
  int m_intervalUSec;
};

} // namespace dss::modbusreader