/*
    Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "businterface-mock.h"
#include "device.h"
#include "model/apartment.h"
#include "model/device-mock.h"

static ds::LogChannel dsLogChannel("deviceTest");
static const char* TAGS = "[model][device]";

using namespace dss;
using namespace trompeloeil;
using namespace std::literals;
using trompeloeil::_;

static const Dsuid dsuid1{"0000000000000000000000000000000001"};

namespace {

TEST_CASE("setSceneValueGR387Workaround", TAGS) {
  auto busInterface = BusInterfaceMock{};
  auto apartment = makeApartment(nullptr, nullptr, &busInterface);

  auto& d = *createDevice(*apartment, {.hardwareType = HardwareType::GR_KL220, .revisionId = 0x387});
  ALLOW_CALL(busInterface.device, getDeviceConfig(_, 129, _)).RETURN(0);

  SECTION("Lower limit") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0));

    d.evaluateGRKL387Workaround(88, 20, 50);
  }

  SECTION("Lower limit+1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0));

    d.evaluateGRKL387Workaround(88, 1, 50);
  }

  SECTION("Upper limit") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x0F));

    d.evaluateGRKL387Workaround(88, 0xffff, 50);
  }

  SECTION("Upper limit-1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x0F));

    d.evaluateGRKL387Workaround(88, 0xfffe, 50);
  }

  SECTION("Exact matching value") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1111, 50);
  }

  SECTION("Exact matching value-1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1110, 50);
  }

  SECTION("Exact matching value-+1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1112, 50);
  }

  SECTION("Intermediate value low") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 6553, 50);
  }

  SECTION("Intermediate value high") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x22));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x22));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, 88, Caller{CallOrigin::system}));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x02));

    d.evaluateGRKL387Workaround(88, 6554, 50);
  }

  SECTION("stateStatuses update") {
    GIVEN("setStateStatuses") {
      std::vector<dsModel::StateStatus> states{};
      states.push_back(dsModel::StateStatus{.id = dsModel::StateId{"1"}, .value = dsModel::StateOptionId{"test1"}});
      states.push_back(dsModel::StateStatus{.id = dsModel::StateId{"2"}, .value = dsModel::StateOptionId{"test2"}});
      d.setStateStatuses(std::move(states));

      CHECK(d.stateStatuses() ==
            std::vector<dsModel::StateStatus>{{.id = dsModel::StateId{"1"}, .value = dsModel::StateOptionId{"test1"}},
                                              {.id = dsModel::StateId{"2"}, .value = dsModel::StateOptionId{"test2"}}});
      THEN("updateStateStatuses") {
        std::vector<dsModel::StateStatus> updatedStates{};
        updatedStates.push_back(
            dsModel::StateStatus{.id = dsModel::StateId{"1"}, .value = dsModel::StateOptionId{"test11"}});
        d.updateStateStatuses(std::move(updatedStates));

        CHECK(d.stateStatuses() == std::vector<dsModel::StateStatus>{
                                       {.id = dsModel::StateId{"1"}, .value = dsModel::StateOptionId{"test11"}},
                                       {.id = dsModel::StateId{"2"}, .value = dsModel::StateOptionId{"test2"}}});
      }
    }
  }
  SECTION("ShadeAttribtes") {
    THEN("windprotection/cardinaldirectoin are initalized") {
      CHECK(d.getCardinalDirection() == cd_none);
      CHECK(d.getWindProtectionClass() == wpc_none);
    }
  }

  SECTION("BinaryInput") {
    GIVEN("akm capable device") {
      auto terminalBlocks = createGRKL300(*apartment, {.hardwareType = HardwareType::GR_KL300});

      WHEN("switch to single input plus binary input") {
        terminalBlocks[0]->setButtonInputMode(ButtonInputMode::STANDARD);
        terminalBlocks[1]->setButtonInputMode(ButtonInputMode::AKM_STANDARD);

        THEN("group memberships are not updated") {
          CHECK(!terminalBlocks[0]->isBinaryInput());
          CHECK(terminalBlocks[1]->isBinaryInput());

          for (auto terminalBlock : terminalBlocks) {
            CHECK(terminalBlock->getButtonID() == ButtonId_Area1);
            CHECK(terminalBlock->getButtonActiveGroup() == GroupIDGray);
            CHECK(terminalBlock->isInGroup(GroupIDGray));
          }
        }

        WHEN("BinaryInput target group is changed") {
          REQUIRE_CALL(busInterface.device, setDeviceConfig(_, CfgClassDevice, 0x40, 8));
          terminalBlocks[1]->setDeviceBinaryInputTargetId(0, GroupIDBlack);
          terminalBlocks[1]->setBinaryInputTargetId(0, GroupIDBlack);

          THEN("2nd is no longer in gray group") {
            CHECK(terminalBlocks[1]->getButtonID() == ButtonId_Area1);
            CHECK(terminalBlocks[1]->getButtonActiveGroup() == GroupIDGray);
            CHECK(!terminalBlocks[1]->isInGroup(GroupIDGray));
            CHECK(terminalBlocks[1]->getActiveGroup() == GroupIDGray); // Bug!
          }
        }
      }
    }
  }
}

} // namespace
