/*
// Copyright (c) 2016 Intel Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
*/

#include <ds/json.h>
#include <ds/log.h>

#include <boost/asio.hpp>
//#include <boost/circular_buffer.hpp>
#include <iostream>

#include "config-file.h"
#include "parser.h"
#include "serial-toolkit.h"
#include "socket-writer.h"

static ds::LogChannel dsLogChannel("p1Main");

namespace dss::p1 {

struct Fuf {
  Fuf(boost::asio::io_service& io, std::chrono::seconds seconds,
      std::function<void(const boost::system::error_code& error_code)>&& callback)
      : m_timer(boost::asio::steady_timer(io, seconds)), m_seconds(seconds), m_callback(std::move(callback)) {}
  ~Fuf() = default;

#if 0
  Fuf(const Fuf&) = default;
  Fuf(Fuf&&) = default;
#endif
  void start() {
    m_timer.expires_at(m_timer.expiry() + m_seconds);
    m_timer.async_wait([this](const auto& ec) -> void { (*this)(ec); });
  }

  void operator()(const boost::system::error_code& error_code) {
    DS_NOTICE("retry timer", m_timer.expiry(), error_code);
    m_callback(error_code);
    start();
  }

  boost::asio::steady_timer m_timer;
  std::chrono::seconds m_seconds;
  std::function<void(const boost::system::error_code& error_code)> m_callback;
};

} // namespace dss::p1

int main(int argc, char** argv) {
  DS_NOTICE("Starting P1 Reader");

  // auto context = ds::ResourceContext{contextData, io, ds::system(), std::this_thread::get_id()};

  auto io = boost::asio::io_service{};

  using dss::p1::loadConfigFile;
  auto config = loadConfigFile(ds::system());

  dss::p1::SenderQueue senderQueue(io, "/var/run/dss-metering.sock");
  dss::p1::MeterParser parser = dss::p1::MeterParser();
  dss::p1::FrameBuffer frameBuffer{senderQueue, config.serialPortConfig.readBufferCapacity, parser};
  dss::p1::SerialConnection serialConnection{io, std::move(config.serialPortConfig), frameBuffer};
  serialConnection.start();

  dss::p1::Fuf reconnectTimer(io, std::chrono::seconds(10), [&serialConnection](const boost::system::error_code& ec) {
    DS_NOTICE("callback", ec);
    serialConnection.start();
  });
  reconnectTimer.start();

  io.run();

  return 0;
}
