#include <boost/filesystem/operations.hpp>
#include <fstream>

#include <ds/catch/catch.h>
#include <ds/clock-mock.h>
#include <ds/log.h>

#include "property-parser.h"
#include "propertysystem.h"

static ds::LogChannel dsLogChannel("propertyParserTest");
static const char* TAGS = "[dss][xml][property-parser]";

using namespace dss;

const std::string TEST_DYNAMIC_DATADIR(ABS_BUILDDIR "/tests/data");
const std::string xmlFile = TEST_DYNAMIC_DATADIR + "/test.xml";

static const char xml[] =
    R"xml(<?xml version="1.0" encoding="utf-8"?>
<properties version="1">
  <property name="bad_property" type="unknown">
    <value></value>
  </property>
  <property name="good_property" type="integer">
    <value>1</value>
  </property>
</properties>)xml";

SCENARIO("Property types are validated when reading from xml storage", TAGS) {
  auto propertyParser = PropertyParser{};

  GIVEN("An apartment.xml with invalid node value type") {
    auto propertySystem = PropertySystem{};
    std::ofstream ofs(xmlFile);
    ofs << xml;
    ofs.close();

    WHEN("Property tree is read") {
      propertyParser.loadFromXML(xmlFile, propertySystem.createProperty("/"));

      THEN("Only properties of valid type will be saved to property tree") {
        REQUIRE_FALSE(propertySystem.getProperty("/bad_property"));
        REQUIRE(propertySystem.getProperty("/good_property"));
      }
    }
    boost::filesystem::remove(xmlFile);
  }
}
