/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "state-change-logger.h"

#include <ds/asio/test.h>
#include <ds/catch/catch.h>
#include <ds/json.h>
#include <ds/log.h>

#include "patch-diff.h"

static ds::LogChannel dsLogChannel("dssStateChangeLoggerTest");
static const char* TAGS = "[dss][StateChangeLogger]";

using namespace dss;
using namespace std::literals;
using trompeloeil::_;

namespace {

struct T {
  int one;
  std::string two;
};

struct S {
  T one;
  std::vector<T> two;
};

struct StateChangeLoggerHooksMock {
  MAKE_MOCK1(patch, void(const Patch&));

  dss::StateChangeLoggerHooks dssStateChangeLoggerHooks() {
    return {.patch = [&](const Patch& arg) { return patch(arg); }};
  }
};

SCENARIO("StateChangeLogger", TAGS) {
  ds::asio::TestLoopResource loop;
  StateChangeLoggerHooksMock hooks{};
  StateChangeLogger stateChangeLogger(loop.io, "foo", hooks.dssStateChangeLoggerHooks());

  SECTION("log change") {
    WHEN("initial change") {
      stateChangeLogger.log(ds::toJsonFragment(S{.one{1, "foo"}}));
      THEN("full state return") {
        /*REQUIRE_CALL(hooks, patch(_))
            .LR_SIDE_EFFECT(CHECK(_1.diff() == std::vector<std::string>{"+{",                          //
                                                                        R"(+  "one":{)",               //
                                                                        R"(+    "one":1,"two":"foo")", //
                                                                        "+  }",                        //
                                                                        "+}",                          //
                                                                        "-null"}));*/
        loop.postStopRun();

        WHEN("second update") {
          stateChangeLogger.log(ds::toJsonFragment(S{.one{2, "bar"}}));
          THEN("only delta reported") {
            /*REQUIRE_CALL(hooks, patch(_))
                .LR_SIDE_EFFECT(CHECK(_1.diff() == std::vector<std::string>{" {",                          //
                                                                            R"(   "one":{)",               //
                                                                            R"(+    "one":2,"two":"bar")", //
                                                                            R"(-    "one":1,"two":"foo")", //
                                                                            "   }",                        //
                                                                            " }"}));*/
            loop.postStopRun();
          }
        }
      }
    }
  }
}

} // namespace

DS_JSON_ANONYMOUS_STRUCT(S, (one)(two));
DS_JSON_ANONYMOUS_STRUCT(T, (one)(two));
