/*
 *  Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#include "state-change-logger.h"

#include <ds/log.h>

#include <algorithm>
#include <numeric>

#include "format-json.h" // json formatter
#include "patch-diff.h"

static ds::LogChannel dsLogChannel{"dssStateChangeLogger"};

namespace dss {

struct StateChangeLogger::Impl {
  boost::asio::io_service& m_io;
  const std::string m_id;
  StateChangeLoggerHooks m_hooks{};
  ds::JsonFragment m_value{}; // -> 'null' as string
  int m_count{};

  void log(ds::JsonFragment&& value) {
    if (dsLogChannel.severity() == ds::LogSeverity::DEBUG) {
      m_io.post([this, value = std::move(value)]() {
        DS_DEBUG(m_id, "change:", ds::hideArg(m_count++));
        auto patch = Patch(formatJson(ds::toJson(value)), formatJson(ds::toJson(m_value)));
        for (const auto& line : patch.diff()) {
          DS_DEBUG(line);
        }
        m_hooks.patch(patch);
        m_value = std::move(value);
      });
    }
  }
};

StateChangeLogger::StateChangeLogger(boost::asio::io_service& io, std::string&& id, StateChangeLoggerHooks hooks)
    : m_impl{new Impl{.m_io = io, .m_id = std::move(id), .m_hooks = std::move(hooks)}} {
  DS_INFO_ENTER(ds::hideArg(m_impl->m_id));
}

StateChangeLogger::~StateChangeLogger() {
  DS_INFO_LEAVE(ds::hideArg(m_impl->m_id));
}

void StateChangeLogger::log(ds::JsonFragment&& value) {
  m_impl->log(std::move(value));
}
} // namespace dss
